// (c) 2015, SAS Institute Inc.

/*global SEA: true, sas: false, sap: false, SC: true, $: false, rs: false, window: false */
/*eslint no-multi-spaces:0, no-underscore-dangle:0 */

// *********************************************************************
// SEA Singleton Objects, In my day they were called globals and we had to walk to school uphill, both ways and we liked it!

// out main top-level object
SEA = {};

// loading async items
SEA.lic =               null;   // what products have been licensed
SEA.userInfo =          null;   // users information
SEA.aRepoTab =          [];     // array of repository tab controls
SEA.aRp =               [];     // array of repositories
SEA.bGetStatus =        null;   // flag to see if we have gotten job info since last login
SEA.prefs =             null;   // have we loaded current users's preferences yet
SEA.iCurrentTab =       0;      // our current repository aka current selected tab index (for 3.1 at least)
SEA.bUIlogging =        false;  // allow logging to be turned on/off from the MT (via remote console for ex.)

SEA.bClosingAllItems = false;

SEA.bClosed = false;    // make sure we only call appClose() once
// saspfj, remove ActJobs, shouldn't be needed now that we have an array of Repositories
SEA.oActJob = null; // no active job yet
SEA.hPDFs = {};     // saspfj, if possible find a better way to implement this
SEA.hDATASETs = {}; // saspfj, if possible find a better way to implement this
SEA.aClaimsData = [];   // start with an empty array, so we know if it's been filled or not
SEA.aEDGs = [];         // start with an empty array, so we know if it's been filled or not
SEA.aActiveJobs = [];   // start with an empty array, so we know if it's been filled or not
SEA._dlgNewJob = null;  // saspfj, is there really no other way???

SEA.bFuture = false;        // future features preview is off by default
SEA.selectedJob = null;
SEA.selectedProj = null;
// for concurrency
SEA.OpenJobID = "";
SEA.OpenParmsID = "";

SEA.EDG = {};
SEA.EDG.oValAndSav = {};
SEA.validationOnly = false;
SEA.ASSOCIATION_SET_ID = "";
SEA.histModel = null;

SEA.buildJobPage = null;
SEA.onConstructEpisodesPress = null;
SEA.onReplaceEpisodesPress = null;
SEA.onClearRepoPress = null;

SEA.handleError = function ( sErrType, sErrMsg ) {
    SEA.log( "!", sErrType + " ERROR: " + sErrMsg );
    sas.hc.m.MessageBox.error( sErrMsg + ". " + oBundle.getText("InternalError.msg"), oBundle.getText("InternalError.title"));
};

SEA.getEmptyRepoCount = function () {
    "use strict";
    var iCnt = 0;
    for ( var x = 0; x < SEA.aRp.length; ++ x ) {
        if ( SEA.aRp[x].vaInfo.status === SC.LASR_STATUS_EMPTY ) {
            ++iCnt;
        }
    }
    SEA.log( "Repo", "Count of Empty Repositories: " + iCnt );
    return iCnt;
};

// keep track of data sources
SEA.dataSources = null;

// new methods nursery
// temp method for testing toast msgs, entries in index.jsp seaSingletons.js and app.js
// does not keep msgs, just briefly displays sent text

// temp testing toast msgs, entries in index.jsp seaSingletons.js and app.js
SEA.popupToast = function ( sTxt, iDelay ) {
    "use strict";
    sas.ea.util.Overview.aSysMsgs.push( sTxt );
    SEA.log( "Toast", sTxt );
    if ( (typeof(iDelay) === SC.UNDEFINED) || (iDelay === null) || (iDelay === 0) ) {
        iDelay = SC.TOAST_TMR;
    }
    $("#popupToastID").html( sTxt );
    $(".toastCSS").stop().fadeIn(iDelay / 10).delay(iDelay).fadeOut(iDelay / 5);
};
SEA.popupUsers = function () {
    "use strict";
    rs.getUsers().resultHandler = function(result) {
        var userInfo = $.parseJSON(result);
        var sOut = "Current Users:\n\n";
        for( var x = 0; x < userInfo.users.length; ++x ) {
            sOut += (userInfo.users[x].userid + ",   " + userInfo.users[x].name + ",   " + userInfo.users[x].status + ",   " + userInfo.users[x].login) + "\n";
        }
        alert(sOut);
    };
};


// saspfj, move all this TIMER stuff to its own file
// timer START *********************************************************************
// Let's see what a timer would look like, so the user knows the elapsed time for step execution
SEA.tmrStep = {};
SEA.tmrStep.aCtrls = [];
SEA.tmrStep.t = null;
SEA.tmrStep.iCount = 0;
SEA.hvChannelCnt = 0;

SEA.tmrStep.startStep = function ( ctrl, iElapsed, sPrefix ) {
    "use strict";

    // with SubSteps we could have more than 1 request to start the same timer
    // so make sure we only allow 1 timer per ctrl
    for ( var x = 0; x < SEA.tmrStep.aCtrls.length; ++x ) {
        if ( ctrl === SEA.tmrStep.aCtrls[x].ctrl ) {
            return;
        }
    }
    //debugger;
    var oTmr = { ctrl: ctrl, iCount: (iElapsed ? -(iElapsed - SEA.tmrStep.iCount) : SEA.tmrStep.iCount), sPrefix: ((typeof sPrefix !== SC.UNDEFINED) ? sPrefix : "") };
    SEA.tmrStep.aCtrls.push( oTmr );
    //console.log( "startStep: " + ctrl.sId + ", " + oTmr.iCount );
    SEA.log( "Timer", "startStep: " + ctrl + ", " + oTmr.iCount );

    // if this is the first control, then let's start the timer
    if (SEA.tmrStep.aCtrls.length === 1) {
        //SEA.tmrStep.iCount = 0;
        SEA.tmrStep.iCount = 0;
        SEA.tmrStep.start();
        SEA.log( "Timer", "TIMER - Started");
    }
};
SEA.tmrStep.endStep = function ( ctrl ) {
    "use strict";

    for ( var x = 0; x < SEA.tmrStep.aCtrls.length; ++x ) {
        // find  ourself
        if ( SEA.tmrStep.aCtrls[x].ctrl === ctrl ) {
            //console.log( "endStep: " + ctrl.sId + ", " + SEA.tmrStep.iCount );
            SEA.log( "Timer", "endStep: " + ctrl + ", " + SEA.tmrStep.iCount );
            // get total time and add 1 second for overhead
            var iCnt = (SEA.tmrStep.iCount - SEA.tmrStep.aCtrls[x].iCount) + 10;

            var sPrefix = SEA.tmrStep.aCtrls[x].sPrefix;

            // this control no longer needs to have a timer
            SEA.tmrStep.aCtrls.splice( x, 1 );

            // update with final time
            //ctrl.setText( SEA.tmrStep.getFriendlyTime(iCnt) );
            if ( sPrefix !== "" ) {
                ctrl.setText( sPrefix + SEA.tmrStep.getFriendlyTime(iCnt, true) );
            } else {
                $(ctrl).html( SEA.tmrStep.getFriendlyTime(iCnt, true) );
            }

            break;  // no need to look any further
        }
    }

    // if no controls left, then stop timer
    if ( SEA.tmrStep.aCtrls.length === 0 ){
        SEA.tmrStep.stop();
        SEA.tmrStep.iCount = 0;
        SEA.log( "Timer", "TIMER - Stopped");
    }
};
SEA.tmrStep.timer = function () {
    "use strict";
    SEA.tmrStep.t = setTimeout( SEA.tmrStep.add, 100 ); // for now, we're counting by tenths/second
};
SEA.tmrStep.start = function () {
    "use strict";
    SEA.tmrStep.timer();
};
SEA.tmrStep.stop = function () {
    "use strict";
    clearTimeout(SEA.tmrStep.t);
};
// return a time string based on tenths of seconds passed in
SEA.tmrStep.getFriendlyTime = function ( iCount, bTenths ) {
    "use strict";
    if ( !bTenths ) {
        iCount *= 10;
    }

    // get tenths & remove them
    var iTenths = Math.round(iCount % 10);
    iCount = Math.floor(iCount / 10);

    // get seconds & remove them
    var iSecs = Math.round(iCount % 60);
    iCount = Math.floor(iCount / 60);

    // get minutes & remove them
    var iMins = Math.round(iCount % 60);
    iCount = Math.floor(iCount / 60);

    // get hours
    var iHrs = Math.round(iCount % 24);

    // remove hours & you're left with # of days
    var iDays = Math.floor(iCount / 24);

    // saspfj, get strings from resource AndOr verify that this is I18N ok

    // this way preserves all leading zeros
    if ( bTenths ) {
        return ( (iDays > 0 ? (iDays + " days - ") : (iHrs < 10 ? "0" : "")) + iHrs + (iMins < 10 ? ":0" : ":") + iMins + (iSecs < 10 ? ":0" : ":") + iSecs + "." + iTenths );
    } else {
        return ( (iDays > 0 ? (iDays + " days - ") : (iHrs < 10 ? "0" : "")) + iHrs + (iMins < 10 ? ":0" : ":") + iMins + (iSecs < 10 ? ":0" : ":") + iSecs );
    }
};
SEA.tmrStep.add = function () {
    "use strict";
    ++SEA.tmrStep.iCount;

    // now update all active controls
    for ( var x = 0; x < SEA.tmrStep.aCtrls.length; ++ x ) {
        //SEA.tmrStep.aCtrls[x].ctrl.setText( SEA.tmrStep.getFriendlyTime( SEA.tmrStep.iCount - SEA.tmrStep.aCtrls[x].iCount ) );

        //$(SEA.tmrStep.aCtrls[x].ctrl).html( SEA.tmrStep.getFriendlyTime( SEA.tmrStep.iCount - SEA.tmrStep.aCtrls[x].iCount, true ) );
        if ( SEA.tmrStep.aCtrls[x].sPrefix !== "" ) {
            SEA.tmrStep.aCtrls[x].ctrl.setText( SEA.tmrStep.aCtrls[x].sPrefix + SEA.tmrStep.getFriendlyTime( SEA.tmrStep.iCount - SEA.tmrStep.aCtrls[x].iCount, true ) );
        } else {
            $(SEA.tmrStep.aCtrls[x].ctrl).html( SEA.tmrStep.getFriendlyTime( SEA.tmrStep.iCount - SEA.tmrStep.aCtrls[x].iCount, true ) );
        }
    }

    SEA.tmrStep.timer();
};
// timer END *********************************************************************


/* saspfj, not used yet
// dialog sizing helpers START *********************************************************************
SEA.dlg = {};
SEA.dlg.getMaxWidth = function () { return $(window).width() - 64; };       // 64 leaves 32 pixel border
SEA.dlg.getMaxHeight = function () { return $(window).height();  - 64; };   // 64 leaves 32 pixel border
// dialog sizing helpers END *********************************************************************
*/

// CONSTANTS - SC
SC = {};

// App Switcher magic strings, taken from SEA\Installs\avaecrmid\Source\Config\Deployment\Metadata\avaecrmid_SoftwareComponent.appxml.org
SC.APPVIEW_JobManager       = "JOB_LIST";
SC.APPVIEW_EDM              = "EPISODE_BUILDER";
SC.APPVIEW_DataLoader       = "DATA_LOADER";
SC.APPVIEW_Associations     = "association";
SC.APPVIEW_EDG              = "episode_definition_group";
SC.APPVIEW_Condition        = "Condition";
SC.APPDIRECTIVE_JobManager    = "JobManagerLogon";
SC.APPDIRECTIVE_EDM           = "EpisodeDefinitionBuilderLogon";
SC.APPDIRECTIVE_DataLoader    = "EpisodeDataLoaderLogon";
SC.APPDIRECTIVE_Associations  = "AssociationLogon";
SC.APPDIRECTIVE_EDG           = "EpisodeDefinitionLogon";
SC.APPDIRECTIVE_Condition     = "ConditionLogon";
SC.LASTVISITVIEW              = "JOB_LIST";

SC.UNDEFINED =              "undefined";            // cause I don't like magic strings all over the place :-(

SC.RETURN_PROJECT =         "ProjectExecutionInfo";
SC.RETURN_SAS =             "SASExecutionInfo";

// NEW
SC.TYPE_STEP =              "Step";
SC.TYPE_SUBSTEP =           "SubStep";
SC.TYPE_FINALSTEP =         "FinalStep";
SC.TYPE_PRINT =             "Print";
SC.TYPE_PATSUB =            "PatientSubset";
SC.TYPE_EXPORT =            "Export";
SC.TYPE_EDGVALIDATION =     "EDGValidation";
//SC.TYPE_QS =                "QuickStats";     // saspfj, future

SC.LASR_STATUS_EMPTY =      "Empty";
SC.LASR_STATUS_USED =       "Populated";
SC.LASR_STATUS_PROCESSING = "Processing";

SC.REPO_STATUS_EMPTY =      "Empty";
SC.REPO_STATUS_WORKING =    "Working";
SC.REPO_STATUS_LOCKED =     "Locked";

SC.EXPORT_STATUS_NONE =     "None";
SC.EXPORT_STATUS_PROCESSING = "Processing";
SC.EXPORT_STATUS_SUCCESS =  "Success";
SC.EXPORT_STATUS_ERROR =    "Error";

SC.JOB_STATUS_NEW =         "New";
SC.JOB_STATUS_WORKING =     "Working";
SC.JOB_STATUS_ARCHIVED =    "Archived";

SC.JOB_TYPE_CONSTRUCTION =  "Construction";
SC.JOB_TYPE_ANALYSIS =      "Analysis";
SC.JOB_TYPE_SUBSET =        "PatientSubset";
SC.JOB_TYPE_TEST =          "Test";

SC.STEP_KEY_EC =            "EC";
SC.STEP_KEY_LV =            "LV";
SC.STEP_KEY_PS =            "PS";
SC.STEP_KEY_PA =            "PA";
SC.STEP_KEY_RA =            "RA";
SC.STEP_KEY_VA =            "VA";
SC.STEP_KEY_GV =            "GV";           // NOTE: hidden step under EC   // saspfj, TBDeleted

SC.JOB_STATE_PENDING =      "Pending";
SC.JOB_STATE_PROCESSING =   "Processing";
SC.JOB_STATE_SUCCESS =      "Success";
SC.JOB_STATE_ERROR =        "Error";

SC.STEP_STATUS_DISABLED =   "Disabled";
SC.STEP_STATUS_PENDING =    "Pending";
SC.STEP_STATUS_PROCESSING = "Processing";
SC.STEP_STATUS_SUCCESS =    "Success";
SC.STEP_STATUS_WARNING =    "Warning";
SC.STEP_STATUS_ERROR =      "Error";

SC.JOB_STATUS_SUCCESS =     "Success";
SC.JOB_STATUS_WARNING =     "Warning";
SC.JOB_STATUS_FAILURE =     "Failure";
SC.JOB_STATUS_CANCELLED =   "Cancelled";
SC.JOB_STATUS_INCOMPLETE =  "Incomplete";
SC.JOB_STATUS_PROCESSING =  "Processing";

SC.VARPTS_EPISODE =         "episode";
SC.VARPTS_PATIENT =         "patient";
SC.VARPTS_PROVIDER =        "provider";
SC.VARPTS_SERVICES =        "services";
SC.VARPTS_BUNDLED =         "bundled";
SC.VARPTS_BUDCOSTS =        "budcosts";
SC.VARPTS_BDPYBDCS =        "bdpybdcs";

// magic numbers & strings
SC.RESULTS_LABEL_HEIGHT =   "64px";
SC.RESULTS_PATH =           "/SASEpisodeAnalytics/rest/files/";
SC.RESULTS_PATH_DATA_LOADER = SC.RESULTS_PATH + "sas";
SC.DSCRPTN_BOX_HEIGHT =     "7em";
SC.COLHDRHEIGHT =           50;     // number determined by Lisa to fix table column headers taking entire page height
SC.MAXMRUCODES =            50;     // max # of MRU codes to keep
SC.MAXHITS =                5000;   // maximum # of hits to put in code picker table
SC.TMR_SLEDGEHAMMER =       200;    // ms delay for sledge hammer approach to overriding default bad behavior of segmented button control
SC.TMR_ASYNC =              200;    // ms timer for checking for completion of async calls for MT setup
SC.TOAST_TMR =              2000;   // default time to display toast


// START ********** for strings used by MidTier services **********

// user states
SC.USER_INACTIVE =          "inactive";
SC.USER_DISCONNECTED =      "disconnected";
SC.USER_ACTIVE =            "active";

// data validation statuses
SC.DATA_VALIDATED =         "01";   // "Validated"
SC.DATA_PENDING =           "02";   // "Pending Validation"
SC.DATA_FAILED =            "03";   // "Load Failed"
SC.DATA_ARCHIVED =          "04";   // "Archived"

// EDG status
SC.EDG_DEV =                "1";    // development
SC.EDG_PROD =               "2";    // production
SC.EDG_RET =                "3";    // retired/inactive

// producers
SC.PROD_EC =                "com.sas.hca.execution.producers.EC";
SC.PROD_SEA =               "com.sas.hca.execution.producers.SEA";
SC.PROD_VAUPLOAD =          "com.sas.hca.execution.producers.VAUpload";

// Batch Request types
SC.METHOD_RUNEC =           "runEC";
SC.METHOD_RUNSTEP =         "runStep";
SC.METHOD_ETL =             "runETL";
SC.METHOD_RUNJOB =          "runJob";
SC.METHOD_SASCODE =         "runSASCode";
SC.METHOD_PRINT =           "procPrint";
SC.METHOD_EXPORT =          "runExport";

// Batch Request run types
SC.PRIORITY_LOW =           "Low";
SC.PRIORITY_NORMAL =        "Normal";
SC.PRIORITY_HIGH =          "High";     // saspfj, 3.2, use high for interactive jobs, like QS?

// system message - Reason
SC.SYSMSG_CONSOLE =         "Console";
SC.SYSMSG_EXCEPTION =       "Exception";
SC.SYSMSG_NOTIFICATION =    "Notification";
SC.SYSMSG_INVALIDCALL =     "InvalidCall";
SC.SYSMSG_LOGON =           "Logon";
SC.SYSMSG_LOGOFF =          "Logoff";
SC.SYSMSG_INFORMATION =     "Information";
SC.SYSMSG_UILOGGINGON =     "UILoggingOn";
SC.SYSMSG_UILOGGINGOff =    "UILoggingOff";


// system message eReason
SC.NOTIFICATION_REPO_CREATED =     "RepositoryCreated";
SC.NOTIFICATION_REPO_MODIFIED =    "RepositoryModified";
SC.NOTIFICATION_REPO_DELETED =     "RepositoryDeleted";
SC.NOTIFICATION_REPO_LOCKED =      "RepositoryLocked";
SC.NOTIFICATION_REPO_REUSED =      "RepositoryReused";
SC.NOTIFICATION_JOB_CREATED =      "JobCreated";
SC.NOTIFICATION_JOB_UPDATED =      "JobUpdated";
SC.NOTIFICATION_JOB_DELETED =      "JobDeleted";
SC.NOTIFICATION_EXPORT_COMPLETED = "ExportCompleted";

// END ********** for strings used by MidTier services **********

// filePath for input data
SC.SEA_FILEPATH = "";
// NEWDIALOGTYPE
SC.NEWDLG_CE = 1;
SC.NEWDLG_RE = 2;
SC.NEWDLG_AJ = 3;



// ---------------------------------------------------------------------
// START  saspfj, tmp area for MT provided objects

// END  saspfj, tmp area for MT provided objects
// ---------------------------------------------------------------------

SEA.iCnt = 0;

SEA.OVRVW = {};
SEA.OVRVW.ID = "OverView";

SEA.sBusyID = "SEA_BUSY_DIALOG";
SEA.iCnt = 0;
SEA.showBusy = function ( sRes ) {
    SEA.log( "BUSY", sRes );

    ++SEA.iCnt;
    // we only need one at a time
    if ( !sap.ui.getCore().byId(SEA.sBusyID) ) {
        SEA.dlgBusy = new sas.hc.m.BusyDialog( SEA.sBusyID, {
            title:            oBundle.getText( sRes ),
            showCancelButton: true, // just in case something happens and the hideBusy() is never called, user has a way out
            close:            function ( event ) {
                SEA.iCnt = 0;   // when it closes, be sure to clear out our use count, used as a fail-safe
            }
        }).open();
    }
};
SEA.hideBusy = function ( sRes ) {
    SEA.log( "BUSY", sRes );

    --SEA.iCnt;
    // if we still have outstanding users, then nothing to do
    if ( SEA.iCnt < 1 ) {
        if ( SEA.dlgBusy ) {
            SEA.dlgBusy.close();
            SEA.dlgBusy.destroy();
            SEA.dlgBusy = null;
        }
        SEA.iCnt = 0;
    }
};

SEA.SLOG = {};
SEA.SLOG.ID = "SEAD";
SEA.SLOG.aKeys = []; SEA.SLOG.aKeys.push( {sKey: "!", bOn: true} );   // ! key is on by default
SEA.SLOG.aMsgs = [];
SEA.SLOG.lStart = Date.now();   // use JS built-in since jQuery isn't loaded yet
SEA.SLOG.bTmr = true;
SEA.SLOG.resetTimer = function () {
    "use strict";
    SEA.SLOG.lStart = $.now();
};
SEA.SLOG.clearLog = function () {
    "use strict";
    SEA.SLOG.aMsgs.splice(0,SEA.SLOG.aMsgs.length);
    SEA.SLOG.refreshLog();
};
SEA.SLOG.timerOn = function (bFlag) {
    "use strict";
    SEA.SLOG.bTmr = bFlag;
};
SEA.SLOG.enableDebug = function () {
    "use strict";
    sap.ui.getCore().byId( SEA.SLOG.ID ).setVisible( true );

    // now refresh the UI
    SEA.SLOG.refreshUI();
};
SEA.SLOG.selectKeys = function ( bFlag ) {
    "use strict";
    for ( var x = 0; x < SEA.SLOG.aKeys.length; ++x ) {
        SEA.SLOG.aKeys[x].bOn = bFlag;
    }
    SEA.SLOG.refreshUI();
};
SEA.SLOG.refreshUI = function () {
    "use strict";

    var tabSEAD = sap.ui.getCore().byId( SEA.SLOG.ID );
    tabSEAD.destroyContent();

    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<span><b>URL: </b>" + window.location + " </span><hr/>" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<button onclick='SEA.SLOG.clearLog();'>Clear Log</button>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<button onclick='SEA.SLOG.refreshUI();'>Refresh UI</button>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<button onclick='SEA.SLOG.refreshLog();'>Refresh Log</button>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<button onclick='SEA.SLOG.resetTimer();'>Reset Timer</button>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<input type='checkbox' " + (SEA.SLOG.bTmr ? "checked" : "") + " onclick='SEA.SLOG.timerOn(this.checked);'>Timer</input>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<button onclick='SEA.SLOG.selectKeys(true);'>All Keys</button>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<button onclick='SEA.SLOG.selectKeys(false);'>No Keys</button>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<input type='checkbox' " + (SEA.lic.EA_ADVANCED ? "checked" : "") + " onclick='SEA.lic.EA_ADVANCED = this.checked;'>Advanced License</input>&nbsp;&nbsp;&nbsp;" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<input type='checkbox' " + (SEA.lic.EA_PQS ? "checked" : "") + " onclick='SEA.lic.EA_PQS = this.checked;'>PQS License</input>&nbsp;&nbsp;&nbsp;" }) );
    //tabSEAD.addContent( new sap.ui.core.HTML({ content: "<a onclick='debugger;' href=''>debugger;</a>" }) );
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<br/><span>Logging Keys:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span>" }) );
    for ( var x = 0; x < SEA.SLOG.aKeys.length; ++x ) {
        tabSEAD.addContent( new sap.ui.core.HTML({
            content: "<input type='checkbox' " + (SEA.SLOG.aKeys[x].bOn ? "checked" : "") + " onclick='SEA.SLOG.setKeyOn(SEA.SLOG.aKeys[" + x + "].sKey,this.checked);'>" + SEA.SLOG.aKeys[x].sKey + "&nbsp&nbsp</input>"
        }) );
    }
    tabSEAD.addContent( new sap.ui.core.HTML({ content: "<h3>Log</h3><div id='vern' style='height:700px; overflow-y: scroll;'>stuff goes here!!!</div><hr/>" }) );

    //new sap.ui.core.HTML({ content:"<h1>Stuff goes here:</h1>" }),
    //new sap.ui.core.HTML({ content:"<h2>Stuff goes here:</h2>" }),
    //new sap.ui.core.HTML({ content:"<h3>Stuff goes here:</h3>" }),
    //new sap.ui.core.HTML({ content:"<h4>Stuff goes here:</h4>" }),
    //new sap.ui.core.HTML({ content:"<h5>Stuff goes here:</h5>" }),
    //new sap.ui.core.HTML({ content:"<h6>Stuff goes here:</h6>" }),

    //tabSEAD.addContent( new sap.ui.core.HTML({ content:"<h3 style='cursor:hand;' onclick='debugger;'>Click for debugger;</h3>" }) );
    //tabVern.addContent( new sap.ui.core.HTML({ content:"" }) );

    // give the UI components time to render before displaying log
    setTimeout( function () {
        SEA.SLOG.refreshLog();
    }, 500 );
};
SEA.SLOG.setKeyOn = function ( sKey, bFlag ) {
    "use strict";
    for ( var x = 0; x < SEA.SLOG.aKeys.length; ++x ) {
        if ( sKey === SEA.SLOG.aKeys[x].sKey ) {
            SEA.SLOG.aKeys[x].bOn = bFlag;
            SEA.SLOG.refreshLog();
            return;
        }
    }
    console.log("setKeyOn: sKey not found");
};
SEA.SLOG.isKeyOn = function ( sKey ) {
    "use strict";
    for ( var x = 0; x < SEA.SLOG.aKeys.length; ++x ) {
        if ( sKey === SEA.SLOG.aKeys[x].sKey ) {
            return SEA.SLOG.aKeys[x].bOn;
        }
    }
    console.log("isKeyOn: sKey not found");
    return false;
};
SEA.SLOG.refreshLog = function () {
    "use strict";
    var sMsgs = "";
    for ( var x = 0; x < SEA.SLOG.aMsgs.length; ++x ) {
        if ( SEA.SLOG.isKeyOn( SEA.SLOG.aMsgs[x].sKey ) ) {
            sMsgs += (SEA.SLOG.aMsgs[x].sMsg + "<br/>");
        }
    }
    $("#vern").html(sMsgs);
};
SEA.log = function ( sKey, sTxt ) {
    "use strict";

    if ( !SEA.bUIlogging ) {
        return;
    }

    // saspfj, ToDo, uncomment when working in 3.2 track

    var bDisplay = true;

    // prune every 2000 lines, only keep the last 2000 lines
    if ( SEA.SLOG.aMsgs.length > 4000 ) {
        SEA.SLOG.aMsgs.splice( 2000, 2000 );
    }

    // do we care about this key?
    for (var x = 0; x < SEA.SLOG.aKeys.length; ++x) {
        if ( sKey === SEA.SLOG.aKeys[x].sKey) {
            if ( SEA.SLOG.aKeys[x].bOn ) {
                // found it & it's on, no need to look further
                break;
            } else {
                // nope, don't display it
                bDisplay = false;
                break;
            }
        }
    }

    // if key wasn't found, then add it
    if ( x >= SEA.SLOG.aKeys.length ) {
        SEA.SLOG.aKeys.push( {sKey: sKey, bOn: true} );
    }

    // UI & MT needs this, so do it once
    var sTime = ($.now() - SEA.SLOG.lStart) / 1000;

    // always log the msg, but only display if user wants it
    var sMsg = sKey + " " + (SEA.SLOG.bTmr ? (sTime + " : ") : "") + sTxt;
    SEA.SLOG.aMsgs.push( {sKey: sKey, sMsg: sMsg} );
    SEA.SLOG.refreshLog();
    if ( bDisplay ) {
        console.log( sMsg );
    }

    // call MT logging
    if ( typeof rs !== SC.UNDEFINED ) {
        rs.writeUILog( sKey, sTime, sTxt ).resultHandler = function () {};    // empty handler so we don't get console errors
    }
};

console.log( "! -- seaSingletons loaded" );


// SEA Singleton objects
// *********************************************************************
